<?php
/*	=====================================================
	lib_setuphelper, Version 1.04.000
	=====================================================

$module_directory     = 'lib_setuphelper';
$module_name          = 'Setup helping library';
$module_function      = 'library';
$module_version       = '1.04.000';
$module_platform      = '2.x';
$module_author        = 'Wolfgang Studer';
$module_license       = 'GNU General Public License';
$module_license_terms = '-';
$module_description   = 'Library providing support on creating SQLs during INSTALL, UPGRADE and UNINSTALL';
$module_guid          = '75de805b-6ee3-47df-91bc-c5746bee12ad';
$module_home          = 'www.wolfgangstuder.ch | www.lepton-cms.com';
$lepador_link         = 'http://www.lepton-cms.com/lepador/library/lib_setuphelper.php';
$forum_link           = 'http://forum.lepton-cms.org/viewtopic.php?f=66&t=3049';

=====================================================

INTRO
==================================

A library module for the LEPTON CMS software.

This module helps for creation of SQL during INSTALL, UPGRADE, UNINSTALL, ADD and DELETE
processes.
Especially during INSTALL and UPGRADE process support for ALTER TABLE is provided.
Also for cleaning up module folder on the file system.

This module supports the module "lib_logger". With "lib_logger",
errors, activities and SQLs can be saved in a database table and shown via the module
"logger".
The modules "lib_logger" and "logger" are optional and not
mandatory for the usage of this module.

This module supports the module "lib_dependency" to secure the availability of
dependencies during de-installations.
The module "lib_dependency" is optional and not mandatory for the usage of this
module.
The module can be installed later on, defined dependencies will be semi-automatically
registered (register_dependency.php).
See README.en.txt of "lib_dependency" for more details about.

PRE-REQUIREMENTS
==================================
>=       2.2 : LEPTON_VERSION
>=       5.6 : PHP_VERSION
>=  0.03.001 : WB_ADDONS[ 'lib_dependency' ] (optional)
>=  0.05.002 : WB_ADDONS[ 'lib_logger' ] (optional)

INSTALLATION
==================================

1. Download the latest version from
	http://www.lepton-cms.com/lepador/modules/lib_setuphelper.php
2. Log into your Lepton CMS backend
3. Go to Addons -> Modules
4. In the "Install Module" section, enter the path to the lib_setuphelper zip file
	that you downloaded in step 1, and choose the "Install" button.


DE-INSTALLATION
==================================

1. Log into your Lepton CMS backend
2. Go to Addons -> Modules
3. In the "Uninstall Module" section, select the module "lib_setuphelper",
	and choose the "Uninstall" button.

USING lib_setuphelper
==================================

This Library-Module can be used in following files:
- install.php
- upgrade.php
- uninstall.php
- add.php
- delete.php

The source content of these files is identical ( Comnets inside are just for documentation
and can be removed !).
Examples of these files can be found in folder "examples" of this library and can be used
and modified in your module.


In addition, a file "_setup.php" is needed in the root of your module folder.
This file contains the table definitions and the respective trigger_code actions.
An examples of this file can be found in folder "examples" of this library and can be used
and modified in your module.


In case the library module "lib_logger" is installed,
you can define the Logging Level:
			// 0 = no logging
			// 1 = errors
			// 4 = infos
			// 5 = errors & infos
			// 8 = notice
			// 9 = errors & notice
			// 12 = infos & notice
			// 13 = all

Following "errors", "infos" and "notice" messages are possible:
- LOG		= Start and End of logging entries. Can not be switched ON/OFF.

- ERROR		= passed trigger is unknow. Allowed triggers are INSTALL, UPGRADE, UNINSTALL, ADD, DELETE.
- ERROR		= passed setup array is empty.
- ERROR		= no field definition in setup array. Occurs during INSTALL, UPGRADE only.
- ERROR		= key 'Field' in fieldset passed in setup array is missing. Occurs during INSTALL, UPGRADE only.
- ERROR		= key 'Type' in fieldset passed in setup array is missing. Occurs during INSTALL, UPGRADE only.
- ERROR		= creation of SHOW CREATE TABLE for temp table during alter table process failed. Occurs during INSTALL, UPGRADE only.
- ERROR		= creation of SHOW CREATE TABLE for existing table during alter table process failed. Occurs during INSTALL, UPGRADE only.
- ERROR		= passed module path does not exists. Occurs only if module folder cleanup is requested.
- ERROR		= archive folder for module folder cleanup could not be created.
- ERROR		= file could not be moved into archive folder during module folder cleanup.
- ERROR		= generic error ocured during SQL execution.

- NOTICE	= no action for current trigger.
- NOTICE	= about if create or alter table process is done. Occurs during INSTALL, UPGRADE only.
- NOTICE	= alter table functionality has been switched off. Occurs during INSTALL, UPGRADE only.
- NOTICE	= no module folder cleanup requested.

- INFO		= SHOW CREATE TABLE content for temp table in alter table process. Occurs during INSTALL, UPGRADE only.
- INFO		= SHOW CREATE TABLE content for existing table in alter table process. Occurs during INSTALL, UPGRADE only.
- INFO		= list of files moved into archive during module folder cleanup.
- INFO		= list of path items where master_index.php has been set/overwritten.
- INFO		= Missing mandatory information when building queries. No error, but query is not beeing executed.
- INFO		= executed query.

#@INCLUDE: README_EN_OPTIONIZER#

#@INCLUDE: README_EN_DEPENDENCY#

OWN ENHANCEMENTS
===========================

This module is build in a way that it can be enhanced by system administrator at various
points without any risk that enhancements gets lost during next updates.
Hereafter the points & files:

Variables used:
- {LEPTON_PATH}   = the LEPTON CMS installation path
- {DEFAULT_THEME} = the backend theme used
- {TEMPLATE}      = the frontend template used
- {COMPONENT}     = the component name, e.g. "modify_optionizer"
- {LANGUAGE}      = the currently used language code, uppercase

- {BACKEND_PATH}  = {LEPTON_PATH}/templates/{DEFAULT_THEME}/backend/lib_setuphelper
- {FRONTEND_PATH} = {LEPTON_PATH}/templates/{TEMPLATE}/frontend/lib_setuphelper


For backend:
- {BACKEND_PATH}/backend.css
	>> use this file to overwrite the default layout styles.
		If this file exists, the default file will not be used!

- {BACKEND_PATH}/languages/{LANGUAGE}.php
	>> use this file to overwrite the default provided text.
		This file is loaded after the standard file have been processed.

- {BACKEND_PATH}/lib/lib.backend.config.{COMPONENT}.php
	>> you can overwrite here default config settings.
		This file is loaded after the standard file have been processed.

- {BACKEND_PATH}/lib/lib.data.{COMPONENT}.php
	>> data have been retrieved. use this script when you want to modify the data before
		they are passed to the TWIG template engine
		This file is loaded after the standard file have been processed.

- {BACKEND_PATH}/lib/lib.twig.{COMPONENT}.php
	>> use this for passing additional values towards the TWIG template.
		This file is loaded after the standard file have been processed.

- {BACKEND_PATH}/templates/backend/{COMPONENT}.lte
	>> use this file to use your own template
		If this file exists, the default file will not be used!
	Note that you can use also a template with a different name by specifying it as:
		// define twig template
		$twigTemplate =  '<<your template name>>';
	in: {BACKEND_PATH}/lib/lib.twig.{COMPONENT}.php





FURTHER DOCUMENTATION
===========================

Further documentation about this module can be found in the LEPTON CMS forum at
http://forum.lepton-cms.org/index.php


Enjoy!
*/
?>
