<?php

/**  // € // to identify files as UTF8
 * Library providing support on creating SQLs during INSTALL, UPGRADE and UNINSTALL
 *
 * @module          lib_setuphelper
 * @author          Wolfgang Studer
 * @copyright       2016 Wolfgang Studer
 * @link            www.LEPTON-cms.org
 * @license         GNU General Public License
 * @license_terms   please see info.php of this module
 * @version         $Id: 1.04.000 | 2016-12-07
 * @lepador_link    http://www.lepton-cms.com/lepador/library/lib_setuphelper.php
 * @forum_link      http://forum.lepton-cms.org/viewtopic.php?f=66&t=3049
*/


// include class.secure.php to protect this file and the whole CMS!
	if ( defined('LEPTON_PATH') == TRUE ):
		include(LEPTON_PATH.'/framework/class.secure.php');
	else:
		$oneback = "../";
		$root = $oneback;
		$level = 1;
		while (( $level < 10 ) && ( file_exists( $root . '/framework/class.secure.php' ) == FALSE )):
			$root .= $oneback;
			$level += 1;
		endwhile;
	
		if ( file_exists($root.'/framework/class.secure.php') == TRUE ):
			include( $root . '/framework/class.secure.php' );
		else:
			trigger_error( sprintf("[ <b>%s</b> ] Can't include class.secure.php!", $_SERVER['SCRIPT_NAME']), E_USER_ERROR );
		endif;
	endif;
// end include class.secure.php


/*	**************************************
	*
	*  DB setup class
	*
	**************************************
*/
class DB_setup
{
	/**
	 *	Class constants.
	 */
	const DBS_NONE		= 0;
	const DBS_ALL		= 13;
	const DBS_ERROR		= E_ERROR;	// 1
	const DBS_NOTICE	= E_NOTICE;	// 8
	const DBS_INFO		= 4;

	/**
	 *	Private var for the current version of this class.
	 */
	private	$version = '$Id: 1.04.000 | 2016-12-07';

	/**
	 *	private var for database object
	 */
	private $database;

	/**
	 *	private var for logger object
	 */
	private $logger;
	private $logger_isset = FALSE;
	private $log_types = array( 'ERROR' => self::DBS_ERROR, 'NOTICE' => self::DBS_NOTICE, 'INFO' => self::DBS_INFO );

	/**
	 *	private var holding the setup array
	 */
	private $setup = array();

	/**
	 *	private var holding the options array
	 */
	private $options = array();

	/**
	 *	private var define the allowed trigger_codes
	 * manual trigger_codes are beeing started from install modules advanced mode!
	 */
	private $allowedtriggers = array( 'INSTALL', 'UPGRADE', 'UNINSTALL', 'ADD', 'DELETE' );
	private $manualtriggers = array( 'MANUAL_INSTALL' => 'INSTALL', 'MANUAL_UPGRADE' => 'UPGRADE' );

	/**
	 *	private var holding errors found in processing
	 */
	private $errors = array();

	/**
	 *	private var holding current settings
	 */
	private $current = array();




	/* ==================================================================================
	 * Constructor of this class
	 * ==================================================================================
	 */
	public function __construct( $database )
	{
		// set database object
		$this->database = $database;

		// init logger object
		$this->setLogger();

		// initiate options array
		$this->setOptions( 'PLACEHOLDERS', 'RESET' );	// used during runquery function
		$this->setOptions( 'TMP_ID', 'RESET' );			// used to generate a dummy table name for alterTable process
		$this->setOptions( 'LOGTEXT', 'RESET' );		// set default log texts
		$this->setOptions( 'MODULE', 'UNKNOWN' );		// set initial value
		$this->setOptions( 'MASTERINDEX', LEPTON_PATH . '/admins/pages/master_index.php' );

		$this->options[ 'MODULEFILES' ]		= array();	// initiate with empty array
		$this->options[ 'WHITELIST' ]		= array();	// initiate with empty array
		$this->options[ 'BLACKLIST' ]		= array();	// initiate with empty array
		$this->options[ 'ARCHIVENAME' ]		= 'setupArchive';	// default
		$this->options[ 'MODULEVERSION' ]	= 0;		// default
	}

	/* ==================================================================================
	 * Destructor of this class
	 * ==================================================================================
	 */
	public function __destruct()
	{
	}


	/* ==================================================================================
	 * specific error handler
	 * target is to handle mkdir & rename warnings as an error
	 *
	 * see also:
	 *http://stackoverflow.com/questions/1241728/can-i-try-catch-a-warning#1241751
	 * ==================================================================================
	 */
	private function throwErrorException( $errstr = null,$code = null, $errno = null, $errfile = null, $errline = null )
	{
		throw new ErrorException($errstr, 0, $errno, $errfile, $errline);
	}
	private function warning_handler( $errno, $errstr, $errfile, $errline, array $errcontext )
	{
		return false && throwErrorException( $errstr, 0, $errno, $errfile, $errline );
		# error_log("AAA"); # will never run after throw
		/* Do execute PHP internal error handler */
		# return false; # will never run after throw
	}

	/* ==================================================================================
	 * Function accepting the logger
	 *
	 * params:	$logger	= object of logger or FALSE
	 * return:	$this
	 * ==================================================================================
	 */
	public function setLogger( $logger = NULL, $logging = 'RESET' )
	{
		if ( is_object( $logger ) == TRUE ):
			$this->logger = $logger;
			$this->logger_isset = TRUE;
			$this->setOptions( 'LOGGING', $logging );		// set passed logging flags
		else:
			$this->logger_isset = FALSE;
			$this->setOptions( 'LOGGING', self::DBS_NONE );	// switch off logging
		endif;

		return $this;
	}


	/* ==================================================================================
	 * Function accepting the setup
	 *
	 * params:	$setup	= array of module setup
	 * return:	$this
	 * ==================================================================================
	 */
	public function setConfig( $setup )
	{
		if ( is_array( $setup ) == TRUE ):
			$this->setup = $setup;
		endif;
	
		return $this;
	}


	/* ==================================================================================
	 * Function accepting the options
	 *
	 * params:	$optionkey	= string for option
	 * params:	$options	= array of module setup
	 * return:	$this
	 * ==================================================================================
	 */
	public function setOptions( $optionkey, $options )
	{
		$optionkey = mb_convert_case( $optionkey, MB_CASE_UPPER, 'UTF-8' );
		switch ( $optionkey ):
		case 'MODULE':
			if ( strlen( trim( $options )) > 0 ):
				$this->options[ $optionkey ] = $options;
			endif;
			break;

		case 'TMP_ID':
			if ( $options == 'RESET' ):
				$this->options[ $optionkey ] = 'XXXXXXXXXXXXXXX';

			elseif (( is_array( $options )		== FALSE )
				and ( strlen( trim( $options ))	> 0      )):
				$this->options[ $optionkey ] = $options;
			endif;
			break;

		case 'PLACEHOLDERS':
			if ( $options == 'RESET' ):
				$this->options[ $optionkey ] = array();

			elseif (( is_array( $options ) == TRUE )
				and ( count( $options ) > 0 )):
				$this->options[ $optionkey ] = $options;
			endif;
			break;

		case 'LOGGING':
			// set logging values
			if ( $this->logger_isset == FALSE ):
				// top priority: no logger instance = no data logging!
				$this->options[ 'LOGGING' ] = array();
				foreach( $this->allowedtriggers as $trigger ):
					$this->options[ 'LOGGING' ][ $trigger ]	= 0;	// Off(0)
				endforeach;

			elseif ( $options == 'RESET' ):
				$this->options[ 'LOGGING' ] = array();
				// first set all to full logging
				foreach( $this->allowedtriggers as $trigger ):
					$this->options[ 'LOGGING' ][ $trigger ]	= self::DBS_ALL;
				endforeach;
				// then reduce
				$this->options[ 'LOGGING' ][ 'ADD' ]		= self::DBS_ERROR;
				$this->options[ 'LOGGING' ][ 'DELETE' ]		= self::DBS_ERROR;

			elseif (( is_array( $options ) == TRUE )
				and ( count( $options )    >  0    )):
				// loop is done so also only parts can be set but only for known triggers
				foreach( $options as $optiontrigger => $optionset ):
					if ( array_key_exists( $optiontrigger, $this->options[ $optionkey ] ) == TRUE ):
						$this->options[ $optionkey ][ $optiontrigger ] = $optionset;
					endif;
				endforeach;

			elseif ( is_array( $options ) == FALSE ):
				// set all
				foreach( $this->options[ $optionkey ] as $optiontrigger => $optionset ):
					$this->options[ $optionkey ][ $optiontrigger ] = $options;
				endforeach;
			endif;
			break;

		case 'LOGTEXT':
			if ( $options == 'RESET' ):
				$options = array();
				$this->options[ $optionkey ] = $options;

			elseif (( is_array( $options ) == TRUE )
				and ( count( $options ) > 0 )):
				$this->options[ $optionkey ] = $options;
			endif;
			break;

		case 'MODULEFILES':
			// list of files belonging to the module
			if (( is_array( $options ) == TRUE )
			and ( count( $options )    >  0    )):
				$this->options[ $optionkey ] = $options;
			endif;
			break;

		case 'WHITELIST':
			// list of files needs to be kept
			if (( is_array( $options ) == TRUE )
			and ( count( $options )    >  0    )):
				$this->options[ $optionkey ] = $options;
			endif;
			break;

		case 'BLACKLIST':
			// list of files to be cleaned up
			if (( is_array( $options ) == TRUE )
			and ( count( $options )    >  0    )):
				$this->options[ $optionkey ] = $options;
			endif;
			break;

		case 'MODULEPATH':
			if (( is_array( $options )		== FALSE )
			and ( strlen( trim( $options ))	>  0     )
			and ( is_dir( $options )		== TRUE  )):
				$this->options[ $optionkey ] = rtrim( $options, DIRECTORY_SEPARATOR ); // remove last slash
			endif;
			break;

		case 'MASTERINDEX':
			if (( is_array( $options )		== FALSE )
			and ( strlen( trim( $options ))	>  0     )
			and ( is_file( $options )		== TRUE  )):
				$this->options[ $optionkey ] = $options;
			endif;
			break;

		case 'ARCHIVENAME':
			if (( is_array( $options )		== FALSE )
			and ( strlen( trim( $options ))	>  0     )):
				$this->options[ $optionkey ] = $options;
			endif;
			break;

		case 'MODULEVERSION':
			if (( is_array( $options )		== FALSE )
			and ( strlen( trim( $options ))	>  0     )):
				$this->options[ $optionkey ] = $options;
			endif;
			break;

		endswitch;

		return $this;
	}


	/* ==================================================================================
	 * Function returning the error array
	 *
	 * return:	$this->errors
	 * ==================================================================================
	 */
	public function getErrors()
	{
		if ( count( $this->errors ) > 0 ):
			return implode( '<br />', $this->errors );
		endif;
	
		return '';
	}


	/* ==================================================================================
	 * Function main processing the trigger_code
	 *
	 * params:	$trigger_code	= defines which trigger_code to be done
	 * return:	TRUE /FALSE (FALSE in case of errors )
	 * ==================================================================================
	 */
	public function process( $trigger_code )
	{
		// trigger_code processing needs uppercase
		$trigger_code = mb_convert_case( $trigger_code, MB_CASE_UPPER, 'UTF-8' );

		// translate manual trigger_codes
		$origintrigger = $trigger_code;
		if ( array_key_exists( $trigger_code, $this->manualtriggers ) == TRUE ):
			$trigger_code = $this->manualtriggers[ $trigger_code ];
		endif;

		// set current array
		$this->current[ 'task_id' ]  = uniqid('', true);	// generate a 99% unique id
		$this->current[ 'trigger_code' ]  = ( $origintrigger == $trigger_code ? $trigger_code : array( $origintrigger, $trigger_code ));
		$this->current[ 'trace' ]    = array( __FUNCTION__ );
		$this->current[ 'add_info' ] = array();
		$this->current[ 'start_ts' ] = date('c');
		// current logging is based on general level
		$this->current[ 'logging' ]  = $this->options[ 'LOGGING' ][ $trigger_code ];

		// start process
		$this->writeLogDetail( array( 'log_type'	=> 'LOG'
									, 'log_code'	=> 'START'
									, 'add_info'	=> 'module version: ' . $this->options[ 'MODULEVERSION' ]
									)
							);


		// doing the process
		$status = $this->doProcess_trigger();

		// regardless of status, set master_index.php to all sub folders
		$this->masterIndex();

		// when status is TRUE and cleanup on filesystem is needed
		// but only for origintrigger INSTALL, UPGRADE
		// Do not do for manualtriggers !!!
		if (( $status									== TRUE      )
		and ( $origintrigger							== 'UPGRADE' )
		and (( count( $this->options[ 'MODULEFILES' ] ) > 0          )
		  or ( count( $this->options[ 'WHITELIST' ] )   > 0         )
		  or ( count( $this->options[ 'BLACKLIST' ] )   > 0         ))):
			$this->current[ 'add_info' ][]	= 'cleanup filesystem';
			$status = $this->fsCleanup();
			array_pop( $this->current[ 'add_info' ] );
		endif;

		// end process
		array_pop( $this->current[ 'add_info' ] );
		$this->current[ 'start_ts' ] = date('c');
		$this->writeLogDetail( array( 'log_type'	=> 'LOG'
									, 'log_code'	=> 'END'
									, 'status'		=> $status
									)
							);

		// close the log
		$this->closeLog();

		// end
		if (( $status			== FALSE )
		 or ( count( $this->errors ) > 0 )):
			return FALSE;
		endif;
		return TRUE;
	}


	/* ==================================================================================
	 * Function processing the trigger_code
	 *
	 * return:	TRUE /FALSE (FALSE in case of errors )
	 * ==================================================================================
	 */
	private function doProcess_trigger()
	{
		// update current array
		$this->current[ 'trace' ][]  = __FUNCTION__;
		$this->current[ 'start_ts' ] = date('c');

		// get current
		$trigger_code = $this->current[ 'trigger_code' ];

		// dummy loop
		while( 1 == 1 ):
			// check if input trigger_code is allowed
			if ( in_array( $trigger_code, $this->allowedtriggers ) == FALSE ):
				$this->writeLogDetail( array( 'log_type'	=> 'ERROR'
											, 'log_code'	=> 'UNKNOWN_TRIGGER'
											, 'status'		=> $status
											)
									);
				$status == FALSE;
				break;
			endif;

			// check if setup array is set
			if ( count( $this->setup ) == 0 ):
				$this->writeLogDetail( array( 'log_type'	=> 'ERROR'
											, 'log_code'	=> 'EMPTY_SETUP'
											, 'status'		=> $status
											)
									);
				$status == FALSE;
				break;
			endif;

			// check if there are data in setup array for current trigger_code
			if ( array_key_exists( $trigger_code, $this->setup ) == FALSE ):
				// trigger_code is not defined in setup
				// this is not an error!
				// just ignore it ....
				$status == TRUE;
				break;
			elseif ( count( $this->setup[ $trigger_code ] ) == 0 ):
				// trigger_code is defined in setup but with empty array
				// this is not an error!
				// just ignore it ....
				$status == TRUE;
				break;
			endif;

			// START TRANSACTION
			$query = 'START TRANSACTION;';
			$status = $this->runQuery( $query );
			if ( $status == FALSE ):
				break;
			endif;

			// start loop tables
			foreach( $this->setup[ $trigger_code ] as $table_name => $data ):

				// per default current logging is based on general level
				$this->current[ 'logging' ]  = $this->options[ 'LOGGING' ][ $trigger_code ];
				// check for logging on table level
				if (( array_key_exists( 'logging', $data )	== TRUE )
				and ( $this->logger_isset					== TRUE )):
					$this->current[ 'logging' ] = $data[ 'logging' ];
				endif;

				// update current array
				$this->current[ 'table_name' ] = $table_name;

				// we assume the default action is to be done when missing
				if ( array_key_exists( 'DO', $data )	== FALSE ):
					$data[ 'DO' ] = TRUE;
				endif;

				// check if there is an DO = FALSE element for current trigger_code on current table
				if (( array_key_exists( 'DO', $data )	== TRUE  )
				and ( $data[ 'DO' ]						== FALSE )):
					// no processing needed
					$this->writeLogDetail( array( 'log_type'	=> 'NOTICE'
												, 'log_code'	=> 'DO_FALSE'
												, 'action'	 	=> 'DO'
												, 'status'		=> $status
											)
									);
					continue;
				endif;

				// processing PRE
				if ( array_key_exists( 'PRE', $data )	== TRUE ):
					// update current array
					$this->current[ 'action' ] = 'PRE';

					$status = $this->doProcess_action_PRE( $data[ 'PRE' ] );
					if ( $status == FALSE ):
						break;	// break table loop
					endif;
				endif;

				// start loop actions
				foreach( $data as $action => $actiondata ):

					// process only DO action
					if ( $action != 'DO' ):
						continue;
					endif;

					// update current array
					$this->current[ 'action' ] = $action;

					// process data depending on trigger_code
					switch ( $trigger_code ):
					case 'INSTALL':
					case 'UPGRADE':
						$status = $this->doProcess_trigger_install( $actiondata );
						break;
					case 'UNINSTALL':
						$status = $this->doProcess_trigger_uninstall( $actiondata );
						break;
					case 'ADD':
						$status = $this->doProcess_trigger_add( $actiondata );
						break;
					case 'DELETE':
						$status = $this->doProcess_trigger_delete( $actiondata );
						break;
					endswitch;
					if ( $status == FALSE ):
						break;	// break action loop
					endif;
				endforeach;

				if ( $status == FALSE ):
					break;	// break table loop
				endif;

				// processing POST
				if ( array_key_exists( 'POST', $data )	== TRUE ):
					// update current array
					$this->current[ 'action' ] = 'POST';

					$status = $this->doProcess_action_POST( $data[ 'POST' ] );
					if ( $status == FALSE ):
						break;	// break table loop
					endif;
				endif;

			endforeach;

			break;	// break dummy loop always
		endwhile;

		// COMMIT or ROLLBACK
		if ( $status == TRUE ):
			$query = 'COMMIT;';
		else:
			$query = 'ROLLBACK;';
		endif;
		$transactionstatus = $this->runQuery( $query );	// do not use $status here !!

		// return processing status
		array_pop( $this->current[ 'trace' ] );
		return $status;
	}


	/* ==================================================================================
	 * Function processing action PRE
	 *
	 * params:	$actiondata	= array with trigger_code table action data
	 * return:	TRUE /FALSE (FALSE in case of errors )
	 * ==================================================================================
	 */
	private function doProcess_action_PRE( $actiondata )
	{
		$status = TRUE;

		// check if there are data available
		if ( count( $actiondata )	> 0 ):
			// update current array
			$this->current[ 'trace' ][]		= __FUNCTION__;
			$this->current[ 'start_ts' ]	= date('c');
			$this->current[ 'add_info' ][]	= 'execute PRE queries';

			$status = $this->execQuery( $actiondata, '' );

			array_pop( $this->current[ 'add_info' ] );
			array_pop( $this->current[ 'trace' ] );
		endif;

		// return processing status
		return $status;
	}


	/* ==================================================================================
	 * Function processing action POST
	 *
	 * params:	$actiondata	= array with trigger_code table action data
	 * return:	TRUE /FALSE (FALSE in case of errors )
	 * ==================================================================================
	 */
	private function doProcess_action_POST( $actiondata )
	{
		$status = TRUE;

		// check if there are data available
		if ( count( $actiondata )	> 0 ):
			// update current array
			$this->current[ 'trace' ][]		= __FUNCTION__;
			$this->current[ 'start_ts' ]	= date('c');
			$this->current[ 'add_info' ][]	= 'execute POST queries';

			$status = $this->execQuery( $actiondata, '' );

			array_pop( $this->current[ 'add_info' ] );
			array_pop( $this->current[ 'trace' ] );
		endif;

		// return processing status
		return $status;
	}


	/* ==================================================================================
	 * PRIVATE Function INSTALL/UPGRADE handling
	 *
	 * params:	$actiondata	= array with trigger_code table action data
	 * return:	TRUE /FALSE (FALSE in case of errors )
	 * ==================================================================================
	 */
	private function doProcess_trigger_install( $actiondata )
	{
		// we process only DO action
		if ( $this->current[ 'action' ] != 'DO' ):
			// nothing to be done
			return TRUE;
		endif;

		$status = TRUE;

		// update current array
		$this->current[ 'trace' ][]		= __FUNCTION__;
		$this->current[ 'start_ts' ]	= date('c');
		$this->current[ 'add_info' ][]	= 'do';

		// get current
		$table_name = $this->current[ 'table_name' ];

		// dummy loop
		while ( 1 == 1 ):

			// is a fieldset available ?
			if (( array_key_exists( 'fields', $this->setup[ $table_name ] )	== TRUE )
			and ( count( $this->setup[ $table_name ][ 'fields' ] )			== 0    )):
				$this->writeLogDetail( array( 'log_type'	=> 'ERROR'
											, 'log_code'	=> 'MISSING_FIELDS'
											, 'status'		=> $status
											)
									);
				$status = FALSE;
				break;
			endif;

			// drop needed ? If yes, do!
			$status = $this->doProcess_trigger_uninstall( $actiondata );
			if ( $status == FALSE ):
				break;
			endif;

			// check if table already exists
			$info = array();
			$error = 0;
			try
			{
				$this->database->describe_table( $table_name, $info );
				if ( $this->database->is_error() == TRUE ):
					$error = $this->database->get_error();
				endif;
			}
			catch  (Exception $e)
			{
				$info = array();
				$error = $e->getMessage();
			}
			// When error, do create table, otherwise alter table
			if ( $error != 0 ):
				$this->writeLogDetail( array( 'log_type'	=> 'NOTICE'
											, 'log_code'	=> 'CHECK'
											, 'add_info'	=> 'do alter table'
											, 'content'		=> $error
											, 'status'		=> $status
											)
									);
			else:
				$this->writeLogDetail( array( 'log_type'	=> 'NOTICE'
											, 'log_code'	=> 'CHECK'
											, 'add_info'	=> 'do create table'
											, 'status'		=> $status
											)
									);
			endif;

			// Create table or upgrade if needed
			if ( count( $info ) == 0 ):
				// create the table
				$this->current[ 'add_info' ][]	= 'create table';
				$status = $this->createTable( $actiondata );
				array_pop( $this->current[ 'add_info' ] );
			else:
				// upgrade the table
				$this->current[ 'add_info' ][]	= 'alter table';
				$status = $this->alterTable( $actiondata, $info );
				array_pop( $this->current[ 'add_info' ] );
			endif;
			if ( $status == FALSE ):
				break;
			endif;

			break;	// break dummy loop always
		endwhile;

		// return
		array_pop( $this->current[ 'trace' ] );
		return $status;
	}


	/* ==================================================================================
	 * PRIVATE Function UNINSTALL handling
	 *
	 * params:	$actiondata	= array with trigger_code table action data
	 * return:	TRUE /FALSE (FALSE in case of errors )
	 * ==================================================================================
	 */
	private function doProcess_trigger_uninstall( $actiondata )
	{
		// we process only DO action
		if ( $this->current[ 'action' ] != 'DO' ):
			// nothing to be done
			return TRUE;
		endif;

		$status = TRUE;

		// update current array
		$this->current[ 'trace' ][]		= __FUNCTION__;
		$this->current[ 'start_ts' ]	= date('c');
		$this->current[ 'add_info' ][]	= 'drop table';

		// get current
		$trigger_code = $this->current[ 'trigger_code' ];
		$table_name = $this->current[ 'table_name' ];

		// DROP = TRUE or FALSE
		$drop = TRUE;	// default for INSTALL, UNINSTALL
		if ( $trigger_code == 'UPGRADE' ):
			$drop = FALSE;	// default for UPGRADE
		endif;
		if ( array_key_exists( 'drop', $this->setup[ $trigger_code ][ $table_name ] )	== TRUE  ):
			$drop = $this->setup[ $trigger_code ][ $table_name ][ 'drop' ];	// overwrite default
		endif;

		// drop the table
		if ( $drop == TRUE ):
			$query = 'DROP TABLE IF EXISTS `' . $table_name . '`;';
			$status = $this->runQuery( $query );
		endif;

		// return status
		array_pop( $this->current[ 'add_info' ] );
		array_pop( $this->current[ 'trace' ] );
		return $status;
	}


	/* ==================================================================================
	 * PRIVATE Function ADD handling
	 *
	 * params:	$actiondata	= array with trigger_code table action data
	 * return:	TRUE /FALSE (FALSE in case of errors )
	 * ==================================================================================
	 */
	private function doProcess_trigger_add( $actiondata )
	{
		// we process only DO action
		if ( $this->current[ 'action' ] != 'DO' ):
			// nothing to be done
			return TRUE;
		endif;

		$status = TRUE;

		// check if there are data available
		if ( count( $actiondata )	> 0 ):
			// update current array
			$this->current[ 'trace' ][]		= __FUNCTION__;
			$this->current[ 'start_ts' ]	= date('c');
			$this->current[ 'add_info' ][]	= 'execute ADD queries';

			$status = $this->execQuery( $actiondata, 'INSERT' );

			array_pop( $this->current[ 'add_info' ] );
			array_pop( $this->current[ 'trace' ] );
		endif;

		// return status
		return $status;
	}


	/* ==================================================================================
	 * PRIVATE Function DELETE handling
	 *
	 * params:	$actiondata	= array with trigger_code table action data
	 * return:	TRUE /FALSE (FALSE in case of errors )
	 * ==================================================================================
	 */
	private function doProcess_trigger_delete( $actiondata )
	{
		// we process only DO action
		if ( $this->current[ 'action' ] != 'DO' ):
			// nothing to be done
			return TRUE;
		endif;

		$status = TRUE;

		// check if there are data available
		if ( count( $actiondata )	> 0 ):
			// update current array
			$this->current[ 'trace' ][]		= __FUNCTION__;
			$this->current[ 'start_ts' ]	= date('c');
			$this->current[ 'add_info' ][]	= 'execute DELETE queries';

			$status = $this->execQuery( $actiondata, 'DELETE' );

			array_pop( $this->current[ 'add_info' ] );
			array_pop( $this->current[ 'trace' ] );
		endif;

		// return status
		return $status;
	}


	/* ==================================================================================
	 * PRIVATE Function create table
	 *
	 * params:	$actiondata	= array with trigger_code table action data
	 * params:	$table_namename	= used in alter table function. When empty, value of $table_name is used insted.
	 * return:	TRUE /FALSE (FALSE in case of errors )
	 * ==================================================================================
	 */
	private function createTable( $actiondata, $tableName = '' )
	{
		$status = TRUE;

		// update current array
		$this->current[ 'trace' ][] = __FUNCTION__;

		// get current
		$trigger_code = $this->current[ 'trigger_code' ];
		$table_name = $this->current[ 'table_name' ];

		// set tableName if not provided
		if ( strlen( $tableName ) == 0 ):
			$tableName = $table_name;
		endif;

		// dummy loop
		while ( 1 == 1 ):

			// build table list for CREATE TABLE
			$lines = array();
			$index = array();
			foreach ( $this->setup[ $table_name ][ 'fields' ] as $key => $fieldset ):
				// build fields array
				$fields = array();
				$status = $this->validateFieldset( $fieldset, $fields );
				if ( $status == FALSE ):
					break;
				endif;

				// combine data elements
				$combination  = '`' . $fields[ 'Field' ] . '`';
				$combination .= ' ' . $fields[ 'Type' ];
				$combination .= ' ' . ( $fields[ 'Null' ] == 'NO' ? 'NOT NULL' : 'NULL' );
				$combination .= ' ' . ( $fields[ 'Default' ] != '' ? 'DEFAULT ' . $fields[ 'Default' ] : '' );
				$combination .= ' ' . ( array_key_exists( 'Extra', $fields ) == TRUE ? $fields[ 'Extra' ] : '' );
				$combination .= ' ' . ( array_key_exists( 'Options', $fields ) == TRUE ? $fields[ 'Options' ] : '' );
				$lines[] = trim( $combination );
			endforeach;

			// abort if an error occured
			if ( $status == FALSE ):
				break;
			endif;

			// build up index lines
			if (( array_key_exists( 'index', $this->setup[ $table_name ] )	== TRUE )
			and ( count( $this->setup[ $table_name ][ 'index' ] )			>  0    )):
				$lines[] = implode( ', ', $this->setup[ $table_name ][ 'index' ] );
			endif;

			// build up table option lines
			if (( array_key_exists( 'options', $this->setup[ $table_name ] )	== TRUE )
			and ( count( $this->setup[ $table_name ][ 'options' ] )			>  0    )):
				$options = implode( ' ', $this->setup[ $table_name ][ 'options' ] );
			endif;

			// build query string and execute
			$query = 'CREATE TABLE ' . $tableName . ' ( ' . implode( ', ', $lines ) . ') ' . $options . ';';
			$status = $this->runQuery( $query );

			break;	// break dummy loop always
		endwhile;

		// return
		array_pop( $this->current[ 'trace' ] );
		return $status;
	}


	/* ==================================================================================
	 * PRIVATE Function alter table
	 *
	 * params:	$actiondata		= array with trigger_code table action data
	 * params:	$info			= DBschema of current table as array
	 * return:	TRUE /FALSE (FALSE in case of errors )
	 * ==================================================================================
	 */
	private function alterTable( $actiondata, $info )
	{
		$status = TRUE;

		// update current array
		$this->current[ 'trace' ][] = __FUNCTION__;

		// get current
		$trigger_code = $this->current[ 'trigger_code' ];
		$table_name = $this->current[ 'table_name' ];

		// dummy loop
		while ( 1 == 1 ):

			// is altering functionality switched OFF ?
			if (( array_key_exists( 'alter', $this->setup[ $trigger_code ][ $table_name ] )	== TRUE )
			and ( $this->setup[ $trigger_code ][ $table_name ][ 'alter' ]					== FALSE )):
				$this->writeLogDetail( array( 'log_type'	=> 'NOTICE'
											, 'log_code'	=> 'CONTENT'
											, 'add_info'	=> 'switched off'
											, 'status'		=> $status
											)
									);
				$status = TRUE;	// return true because this is no failure but feature
				break;	
			endif;

			// define temp table name
			$table_nameTmp = TABLE_PREFIX . 'tmp_' . $this->options[ 'TMP_ID' ];

			// drop temp table if exists
			$query = 'DROP TABLE IF EXISTS `' . $table_nameTmp . '`';
			$status = $this->runQuery( $query );
			if ( $status == FALSE ):
				break;	
			endif;

			// now get AUTO_INCREMENT counter of existing table if exists
			$query = 'SELECT AUTO_INCREMENT FROM information_schema.TABLES where TABLE_NAME = "' . $table_name . '"';
			$ai = $this->database->get_one( $query );
				if (( is_numeric( $ai ) == TRUE )
				and ( $ai               >  0    )):
					$this->setup[ $table_name ][ 'options' ][] = 'AUTO_INCREMENT=' . $ai;
				endif;

			// then create a temp table with new setup
			$status = $this->createTable( $actiondata, $table_nameTmp );
			if ( $status == FALSE ):
				break;	
			endif;

			// now get SHOW CREATE TABLE result for this newly created 'temp' table
			try
			{
				$query = 'SHOW CREATE TABLE `' . $table_nameTmp . '`;';
				$handler = $this->database->execute_query(
						$query,
						true,
						$showTableDef
					);
			}
			catch  (Exception $e)
			{
				$status = FALSE;
			}
			if ( $status == FALSE ):
				$this->writeLogDetail( array( 'log_type'	=> 'ERROR'
											, 'log_code'	=> 'FAILED'
											, 'add_info'	=> 'SHOW CREATE TABLE /-> ' . $table_nameTmp
											, 'status'		=> $status
											)
									);
				break;	
			else:
				// we need to replace the temp table name
				$showTableNew = str_replace( $table_nameTmp, $table_name, $showTableDef[0][ 'Create Table' ] );

				$this->writeLogDetail( array( 'log_type'	=> 'INFO'
											, 'log_code'	=> 'CONTENT'
											, 'add_info'	=> 'SHOW CREATE TABLE /-> ' . $table_nameTmp
											, 'content'		=> $showTableNew
											, 'status'		=> $status
											)
									);
			endif;

			// also get SHOW CREATE TABLE result for already existing table
			try
			{
				$query = 'SHOW CREATE TABLE `' . $table_name . '`;';
				$handler = $this->database->execute_query(
						$query,
						true,
						$showTableDef
					);
			}
			catch  (Exception $e)
			{
				$status = FALSE;
			}
			if ( $status == FALSE ):
				$this->writeLogDetail( array( 'log_type'	=> 'ERROR'
											, 'log_code'	=> 'FAILED'
											, 'add_info'	=> 'SHOW CREATE TABLE /-> ' . $table_name
											, 'status'		=> $status
											)
									);
				break;	
			else:
				$showTableOld = $showTableDef[0][ 'Create Table' ];

				$this->writeLogDetail( array( 'log_type'	=> 'INFO'
											, 'log_code'	=> 'CONTENT'
											, 'add_info'	=> 'SHOW CREATE TABLE /-> ' . $table_name
											, 'content'		=> $showTableOld
											, 'status'		=> $status
											)
									);
			endif;

			// we can now compare both table definitions
			if ( $showTableNew == $showTableOld ):
				// we can drop the temp table
				$query = 'DROP TABLE IF EXISTS `' . $table_nameTmp . '`;';
				$status = $this->runQuery( $query );
				if ( $status == FALSE ):
					break;	
				endif;

			else:
				// count number of rows in existing table
				$query = 'SELECT count(*) FROM `' . $table_name . '`;';
				$count = $this->database->get_one( $query );
				if (( is_numeric( $count ) == TRUE )
				and ( $count               >  0    )):
			
					// we have to build an insert sql to transfer data from existing to new table
					$targetlist = array();
					$sourcelist = array();
					// copy only fields existing in both old & new
					foreach ( $info as $infokey => $infoset ):
						// does source field exists in new setup
						$fields = array();
						foreach ( $this->setup[ $table_name ][ 'fields' ] as $fieldkey => $fieldset ):
							if (( $infoset[ 'Field' ] == $fieldset[ 'Field' ] )
							 or ( strlen( array_search( $infoset[ 'Field' ], array_column( $fieldset, 'Source' )) ) > 0 )):
								$fields = $fieldset;
								break;
							endif;
						endforeach;

						// field found
						if ( count( $fields ) > 0 ):
							// check if confition exists & match
							if ( array_key_exists( 'Convert', $fields ) == FALSE ):
								// source & target are same field name
								$source = '`' . $fields[ 'Field' ] . '`';
							elseif ( array_key_exists( 'Source', $fields[ 'Convert' ] ) == FALSE ):
								// convert exists but incomplete, so use also target field as source
								$source = '`' . $fields[ 'Field' ] . '`';
							else:
								// convert exists and is valid
								$source = '`' . $fields[ 'Convert' ][ 'Source' ] . '`';
								if ( array_key_exists( 'Rule', $fields[ 'Convert' ] )   == TRUE  ):
									if (( array_key_exists( 'onType', $fields[ 'Convert' ] ) == FALSE )
									 or ( $infoset[ 'Type' ]    ==   $fields[ 'Convert' ][ 'onType' ] )):
										// convert exists but no rule and no condition, so rule is used always
										// condition exists & match
										// if condition would not match then use Source field value only
										$source = $fields[ 'Convert' ][ 'Rule' ];
									endif;
								endif;
							endif;

							// build processing list
							$sourcelist[] = $source;
							$targetlist[] = $fields[ 'Field' ];
						endif;
					endforeach;

					// insert into xyz(t_id,v_id,f_name) select t_id,v_id,f_name from xyz2;
					$query = 'INSERT INTO `' . $table_nameTmp . '` ( `';
					$query .= implode( '`, `', $targetlist );
					$query .= '` ) SELECT ';
					$query .= implode( ', ', $sourcelist );
					$query .=' FROM `' . $table_name . '`;';

					// run the data transfering query
					$status = $this->runQuery( $query );
					if ( $status == FALSE ):
						break;	
					endif;

					// rename or drop existing table
					if (( array_key_exists( 'bak_copy', $this->setup[ $trigger_code ][ $table_name ] )	== FALSE )
					 or ( $this->setup[ $trigger_code ][ $table_name ][ 'bak_copy' ]						== TRUE )):
						// build bak copy table name
						$table_nameBak = $table_name . '_BAK_' . date('YmdHis');

						// create secure copy
						$query = 'RENAME TABLE  `' . $table_name . '` TO  `' . $table_nameBak . '`;';
					else:
						// drop the old table
						$query = 'DROP TABLE IF EXISTS `' . $table_name . '`;';
					endif;
					$status = $this->runQuery( $query );
					if ( $status == FALSE ):
						break;	
					endif;

				// existing table is empty: Drop it!
				else:
					$query = 'DROP TABLE IF EXISTS `' . $table_name . '`;';
					$status = $this->runQuery( $query );
					if ( $status == FALSE ):
						break;	
					endif;
				endif;

				// rename temp table to default table name
				$query = 'RENAME TABLE  `' . $table_nameTmp . '` TO  `' . $table_name . '`;';
				$status = $this->runQuery( $query );
				if ( $status == FALSE ):
					break;	
				endif;
			endif;

			break;	// break dummy loop always
		endwhile;

		// return
		array_pop( $this->current[ 'trace' ] );
		return $status;
	}


	/* ==================================================================================
	 * PRIVATE Function handle filesystem cleanup
	 *
	 * return:	TRUE /FALSE (FALSE in case of errors )
	 * ==================================================================================
	 */
	private function fsCleanup()
	{
		$status = TRUE;

		// update current array
		$this->current[ 'trace' ][] = __FUNCTION__;

		// get current
		$trigger_code = $this->current[ 'trigger_code' ];

		// dummy loop
		while ( 1 == 1 ):

			// is cleanup on filesystem needed ?
			if (( count( $this->options[ 'MODULEFILES' ] ) == 0 )
			and ( count( $this->options[ 'WHITELIST' ] )   == 0 )
			and ( count( $this->options[ 'BLACKLIST' ] )   == 0 )):
				$this->writeLogDetail( array( 'log_type'	=> 'NOTICE'
											, 'log_code'	=> ''
											, 'add_info'	=> 'No file system cleanup requested'
											, 'status'		=> $status
											)
									);
				// this is no error
				break;
			endif;

			if ( is_dir( $this->options[ 'MODULEPATH' ] ) == FALSE ):
				$this->writeLogDetail( array( 'log_type'	=> 'ERROR'
											, 'log_code'	=> ''
											, 'add_info'	=> 'Modulepath is wrong or missing'
											, 'status'		=> $status
											, 'content'		=> $this->options[ 'MODULEPATH' ]
											)
									);
				$status = FALSE;
				break;
			endif;

			// identify obsolete files for archive
			$obsoleteFiles = array();
			$dirs = array();

			// initiate list of obsolete files with items from blacklist
			if ( count( $this->options[ 'BLACKLIST' ] ) > 0 ):
				// loop over blacklist items.
				// each item must start with LEPTON_PATH in obsolete files list
				// only existing files & folders are added
				foreach ( $this->options[ 'BLACKLIST' ] as $itemName ):
					// check if item does not start with LEPTON_PATH
					if ( strpos( $itemName, LEPTON_PATH, 0 ) == FALSE ):
						// remove slash in first char
						$itemName = ltrim( $itemName, DIRECTORY_SEPARATOR );
						// add LEPTON PATH
						$itemName = LEPTON_PATH . DIRECTORY_SEPARATOR . $itemName;
					endif;

					// update dirs list
					$this->setDirs( $itemName, 0, $dirs );

					// add to obsolete list if file exists
					if ( is_file( $itemName ) == TRUE ):
						$obsoleteFiles[] = $itemName;
					endif;
				endforeach;
			endif;

			// add files not belonging to list of module files
			if ( count( $this->options[ 'MODULEFILES' ] ) > 0 ):
				// loop over module path
				// each item must start with LEPTON_PATH in obsolete files list
				// only existing files & folders are added
				foreach ( new RecursiveIteratorIterator( new RecursiveDirectoryIterator( $this->options[ 'MODULEPATH' ] )) as $itemName ):

					// compare with archive name
					$checkFile  = str_replace( LEPTON_PATH, '', $itemName );

					if ( is_file( $itemName ) == TRUE ):

						// check if file is inside the list of MODULEFILES
						if ( in_array( $itemName, $this->options[ 'MODULEFILES' ] )	== TRUE ):
							$plusdir = 1;

						// setup Archives will be kept by adding into whitelist
						elseif ( strpos( mb_convert_case( $checkFile, MB_CASE_LOWER, 'UTF-8' ), mb_convert_case( $this->options[ 'ARCHIVENAME' ], MB_CASE_LOWER, 'UTF-8' ) ) == TRUE ):
							$this->options[ 'WHITELIST' ][] = $itemName;
							$plusdir = 1;

						// ignore index files
						elseif ( strpos( mb_convert_case( $itemName, MB_CASE_LOWER, 'UTF-8' ), 'index.php' )		== TRUE ):
							// does not impact dirslist
							$plusdir = 0;

						else:
							// file is obsolete
							$obsoleteFiles[] = $itemName;
							$plusdir = 0;
						endif;

						// update dirs list
						$this->setDirs( $itemName, $plusdir, $dirs );
					endif;
				endforeach;
			endif;

			// make list of obsolete files unique
			$tmp = array_unique( $obsoleteFiles );
			$obsoleteFiles = $tmp;
			unset( $tmp);

			// remove files from whitelist out of list of obsolete files
			if ( count( $this->options[ 'WHITELIST' ] ) > 0 ):
				// loop over blacklist items.
				// each item must start with LEPTON_PATH in obsolete files list
				// only existing files & folders are added
				foreach ( $this->options[ 'WHITELIST' ] as $itemName ):
					// check if item does not start with LEPTON_PATH
					if ( strpos( $itemName, LEPTON_PATH, 0 ) == FALSE ):
						// remove slash in first char
						$itemName = ltrim( $itemName, DIRECTORY_SEPARATOR );
						// add LEPTON PATH
						$itemName = LEPTON_PATH . DIRECTORY_SEPARATOR . $itemName;
					endif;

					// update dirs list
					$this->setDirs( $itemName, 1, $dirs );

					// remove from obsolete list if file exists
					// note that list of obsolete files does not contains folders
					if ( is_file( $itemName ) == TRUE ):
						$key = array_search( $itemName, $obsoleteFiles  );
						if ( $key !== false ):
							unset($array[$key]);
						endif;
					endif;
				endforeach;
			endif;

			// set specific error handler to suppress warnings
			set_error_handler( array( $this, 'warning_handler' ) );

			// archive only if obsolete files have been found
			if ( count( $obsoleteFiles ) > 0 ):

				// create an archive folder
				$archivePath = $this->options[ 'MODULEPATH' ] . DIRECTORY_SEPARATOR . $this->options[ 'ARCHIVENAME' ] . '_' . date('YmdHis');
				$status = $this->createFolder( $archivePath );
				if ( $status == FALSE ):
					break;	// break dummy loop
				endif;

				// loop over array of dirs and move those to archive
				// which does not contain files to be kept
				foreach ( $dirs as $dir => $dircount ):
					if (( $dircount == 0 ) and ( strlen( $dir ) > 1 )):
						// move dir to archive
						$source = $this->options[ 'MODULEPATH' ] . $dir;
						if ( is_dir( $source ) == TRUE ):
							$target = $archivePath . DIRECTORY_SEPARATOR . $dir;

							// create target dir
							if ( is_dir( $target ) == FALSE ):
								$status = $this->createFolder( $target );
								if ( $status == FALSE ):
									break;	// break dummy loop
								endif;
							endif;

							// move the folder
							$status = $this->moveFile( $source, $target );
						endif;
					endif;
				endforeach;
				if ( $status == FALSE ):
					break;	// break dummy loop
				endif;

				// loop over array of obsolete files and move files if still exists (may have been moved with dirs)
				foreach ( $obsoleteFiles as $file ):
					if ( is_file( $file ) == TRUE ):
						$moduleFile = str_replace( $this->options[ 'MODULEPATH' ] . DIRECTORY_SEPARATOR, '', $file );
						// move file to archive
						$source = $file;
						if ( is_file( $source ) == TRUE ):
							$target = $archivePath . DIRECTORY_SEPARATOR . $moduleFile;

							// create target dir
							if ( is_dir( dirname( $target )) == FALSE ):
								$status = $this->createFolder( dirname( $target ));
								if ( $status == FALSE ):
									break;	// break dummy loop
								endif;
							endif;

							// move the file
							$status = $this->moveFile( $source, $target );
						endif;
					endif;
				endforeach;
				if ( $status == FALSE ):
					break;	// break dummy loop
				endif;

				// finally loop over archive folder and build final list of obsolete files & folders
				$obsoleteFiles = array();
				foreach ( new RecursiveIteratorIterator( new RecursiveDirectoryIterator( $archivePath )) as $itemName ):
					$obsoleteFiles[] = str_replace( LEPTON_PATH, '', $itemName );
				endforeach;
			endif;

			// restore default error handler
			restore_error_handler();

			// write summary
			$this->writeLogDetail( array( 'log_type'		=> 'INFO'
											, 'log_code'	=> 'CONTENT'
											, 'add_info'	=> 'cleanup files: ' . count( $obsoleteFiles )
											, 'content'		=> implode( ', ', $obsoleteFiles )
											, 'status'		=> $status
											)
									);

			break;	// break dummy loop always
		endwhile;

		// return
		array_pop( $this->current[ 'trace' ] );
		return $status;
	}


	/* ==================================================================================
	 * PRIVATE Function update list of folders
	 *
	 * params:	$path		= path to be created
	 * return:	TRUE /FALSE (FALSE in case of errors )
	 * ==================================================================================
	 */
	private function setDirs( $itemName, $plusdir, &$dirs )
	{
		$status = TRUE;

		// update current array
		$this->current[ 'trace' ][] = __FUNCTION__;

		// dirs list contains only path elements after module path
		$dbs_modulePath = rtrim( str_replace( $this->options[ 'MODULEPATH' ], '', dirname( $itemName ) ), DIRECTORY_SEPARATOR );
		if ( strlen( trim( $dbs_modulePath )) == 0 ):
			$dbs_modulePath = '.';
		endif;

		// add to dirs list
		if ( array_key_exists( $dbs_modulePath, $dirs ) == TRUE ):
			$dirs[ $dbs_modulePath ] += $plusdir;
		else:
			// add each item path to dirs
			$arr = explode( DIRECTORY_SEPARATOR, $dbs_modulePath );
			$chk = array();
			foreach ( $arr as $item ):
				if ( strlen( trim( $item )) == 0 ):
					$item = '.';
				endif;
				$chk[] = $item;
				$tmp = implode( DIRECTORY_SEPARATOR, $chk );
				if ( array_key_exists( $tmp, $dirs ) == FALSE ):
					$dirs[ $tmp ] = $plusdir;
				endif;
			endforeach;
		endif;

		// return
		array_pop( $this->current[ 'trace' ] );
		return $status;
	}


	/* ==================================================================================
	 * PRIVATE Function creating a folder
	 *
	 * params:	$path		= path to be created
	 * return:	TRUE /FALSE (FALSE in case of errors )
	 * ==================================================================================
	 */
	private function createFolder( $path )
	{
		$status = TRUE;

		// update current array
		$this->current[ 'trace' ][] = __FUNCTION__;

		try
		{
			mkdir( $path , 0755, TRUE );
		}
		catch (Exception $e)
		{
			$error = $e->getMessage();
			$status = FALSE;
		}
		if ( $status == FALSE ):
			$this->writeLogDetail( array( 'log_type'	=> 'ERROR'
										, 'log_code'	=> 'FAILED'
										, 'add_info'	=> 'on mkdir'
										, 'status'		=> $status
										, 'content'		=> 'mkdir( ' . $path . ' , 0755, TRUE )'
										)
								);
		endif;

		// return
		array_pop( $this->current[ 'trace' ] );
		return $status;
	}


	/* ==================================================================================
	 * PRIVATE Function move a file or folder
	 *
	 * params:	$source		= source file/folder name to be moved
	 * params:	&$fields	= target destination
	 * return:	TRUE /FALSE (FALSE in case of errors )
	 * ==================================================================================
	 */
	private function moveFile( $source, $target )
	{
		$status = TRUE;

		// update current array
		$this->current[ 'trace' ][] = __FUNCTION__;

		// echo 'move: <br/>' . $source . '<br />' . $target . '<br /><br />';
		try
		{
			rename( $source, $target );
		}
		catch (Exception $e)
		{
			$error = $e->getMessage();
			$status = FALSE;
		}
		if ( $status == FALSE ):
			$this->writeLogDetail( array( 'log_type'	=> 'ERROR'
										, 'log_code'	=> 'FAILED'
										, 'add_info'	=> 'on move'
										, 'status'		=> $status
										, 'content'		=> 'rename( "' . $source . '", "' . $target . '" ) -> ' . $error
										)
								);
		endif;

		// return
		array_pop( $this->current[ 'trace' ] );
		return $status;
	}


	/* ==================================================================================
	 * PRIVATE Function add master index.php into module path and all sub folders
	 *
	 * return:	TRUE /FALSE (FALSE in case of errors )
	 * ==================================================================================
	 */
	private function masterIndex()
	{
		$status = TRUE;

		// update current array
		$this->current[ 'trace' ][] = __FUNCTION__;

		// loop over all folders of module installation
		$indexList = array();
		$master_index = $this->options[ 'MASTERINDEX' ];

		if ( is_file( $master_index ) == TRUE ):
			$lastItem = '';
			foreach ( new RecursiveIteratorIterator( new RecursiveDirectoryIterator( $this->options[ 'MODULEPATH' ] )) as $itemName ):
				if ( is_dir( $itemName ) == TRUE ):
					// remove dots
					if ( strpos( $itemName, DIRECTORY_SEPARATOR . '..' ) >= 0 ):
						$itemName = str_replace( DIRECTORY_SEPARATOR . '..', '', $itemName );
					endif;
					if ( strpos( $itemName, DIRECTORY_SEPARATOR . '.' ) >= 0 ):
						$itemName = str_replace( DIRECTORY_SEPARATOR . '.', '', $itemName );
					endif;

					// item appears twice: 1) one dot, 2) two dots.
					// it is needed only once.
					if ( $itemName == $lastItem ):
						continue;
					endif;
					$lastItem = $itemName;

					// copy master index.php into this folder, overwriting any existing
					$targetPathFile = $itemName . DIRECTORY_SEPARATOR . 'index.php';
					copy( $master_index, $targetPathFile );

					$indexList[] = $targetPathFile;
				endif;
			endforeach;
		endif;

		// write summary
		$this->writeLogDetail( array( 'log_type'		=> 'INFO'
										, 'log_code'	=> 'CONTENT'
										, 'add_info'	=> 'set master index: ' . count( $indexList )
										, 'content'		=> 'copy ' . $master_index . ' to ( ' . implode( ', ', $indexList ) .' )'
										, 'status'		=> $status
										)
								);

		// return
		array_pop( $this->current[ 'trace' ] );
		return $status;
	}


	/* ==================================================================================
	 * PRIVATE Function validate fields
	 *
	 * params:	$fieldset		= source array from setup
	 * params:	&$fields		= target array
	 * return:	TRUE /FALSE (FALSE in case of errors )
	 * ==================================================================================
	 */
	private function validateFieldset( $fieldset, &$fields )
	{
		$status = TRUE;

		// update current array
		$this->current[ 'trace' ][] = __FUNCTION__;

		// build field array
		$fields = array();

		// dummy loop
		while ( 1 == 1 ):

			if ( array_key_exists( 'Field', $fieldset )	== FALSE  ):
				// Field is mandatory
				$this->writeLogDetail( array( 'log_type'	=> 'ERROR'
											, 'log_code'	=> 'MISSING_MANDATORY'
											, 'add_info'	=> 'validate Fields / Key: Field'
											, 'status'		=> $status
											)
									);
				$status = FALSE;
				break;
			endif;
			$fields[ 'Field' ]	= $fieldset[ 'Field' ];

			if ( array_key_exists( 'Type', $fieldset )	== FALSE  ):
				// Type is mandatory
				$this->writeLogDetail( array( 'log_type'	=> 'ERROR'
											, 'log_code'	=> 'MISSING_MANDATORY'
											, 'add_info'	=> 'validate Fields / Key: Type'
											, 'status'		=> $status
											)
									);
				$status = FALSE;
				break;
			endif;
			$fields[ 'Type' ]	= $fieldset[ 'Type' ];

			if (( array_key_exists( 'Null', $fieldset )	== TRUE )
			and ( $fieldset[ 'Null' ]					== 'NO' )):
				$fields[ 'Null' ]	= 'NO';
			else:
				$fields[ 'Null' ]	= 'YES';
			endif;

			if (( array_key_exists( 'Default', $fieldset )	== TRUE )
			and ( strlen( $fieldset[ 'Default' ] )			>  0    )):
				$fields[ 'Default' ]	= $fieldset[ 'Default' ];
			else:
				$fields[ 'Default' ]	= '';
			endif;

			if (( array_key_exists( 'Extra', $fieldset )	== TRUE )
			and ( strlen( $fieldset[ 'Extra' ] )			>  0    )):
				$fields[ 'Extra' ]	= $fieldset[ 'Extra' ];
			else:
				$fields[ 'Extra' ]	= '';
			endif;

			if (( array_key_exists( 'Key', $fieldset )	== TRUE )
			and ( strlen( $fieldset[ 'Key' ] )			>  0    )):
				$fields[ 'Key' ]	= $fieldset[ 'Key' ];
			else:
				$fields[ 'Key' ]	= '';
			endif;

			if (( array_key_exists( 'Options', $fieldset )	== TRUE )
			and ( strlen( $fieldset[ 'Options' ] )			>  0    )):
				$fields[ 'Options' ]	= $fieldset[ 'Options' ];
			else:
				$fields[ 'Options' ]	= '';
			endif;

			break;	// break dummy loop always
		endwhile;

		// return
		array_pop( $this->current[ 'trace' ] );
		return $status;
	}


	/* ==================================================================================
	 * PRIVATE Function preparing & execution queries
	 *
	 * params:	$actiondata		= array with trigger_code table action data
	 * params:	$baserequest	= the request like PRE, POST, INSERT, UPDATE, ..
	 * return:	TRUE /FALSE (FALSE in case of errors )
	 * ==================================================================================
	 */
	private function execQuery( $actiondata, $baserequest )
	{
		$status = TRUE;

		// update current array
		$this->current[ 'trace' ][] = __FUNCTION__;

		// get current
		$table_name = $this->current[ 'table_name' ];

		// process input
		foreach( $actiondata as $datakey => $dataitem ):
			$request		= $baserequest;
			$fields			= array();
			$where			= array();
			$query			= '';
			if ( array_key_exists( 'REQUEST', $dataitem )	== TRUE ):
				$request  = $dataitem[ 'REQUEST' ];
			endif;
			if ( array_key_exists( 'TABLE', $dataitem )	== TRUE ):
				$table_name  = $dataitem[ 'TABLE' ];
			endif;
			if ( array_key_exists( 'FIELDS', $dataitem )	== TRUE ):
				$fields  = $dataitem[ 'FIELDS' ];
			endif;
			if ( array_key_exists( 'WHERE', $dataitem )	== TRUE ):
				$where   = $dataitem[ 'WHERE' ];
			endif;
			if ( array_key_exists( 'QUERY', $dataitem )	== TRUE ):
				$query   = $dataitem[ 'QUERY' ];
			endif;

			$status = TRUE;
			if ( strlen( $request ) > 0 ):
				$build = TRUE;
				switch ( mb_convert_case( $request, MB_CASE_UPPER, 'UTF-8' )):
				case 'DELETE':
					break;

				case 'UPDATE':
				case 'INSERT':
				case 'SELECT':
				case 'ALTER':
					if ( count( $fields ) == 0 ):
						$build = FALSE;
					endif;
					break;

				default:
					// request is invalid
					$build == FALSE;
				endswitch;

				// read next
				if ( $build == FALSE ):
					$this->writeLogDetail( array( 'log_type'	=> 'INFO'
												, 'log_code'	=> 'MISSING_MANDATORY'
												, 'add_info'	=> 'build and execute queries'
												, 'content'		=> json_encode( $dataitem )
												, 'status'		=> $status
												)
										);
					continue;
				endif;

				if ( is_array( $where ) == TRUE ):
					$where = $this->buildWhere( $where );
				endif;

				$query = $this->database->build_mysql_query(
							  $request
							, $table_name
							, $fields
							, $where
							);
			endif;

			// run query
			if ( strlen( $query ) > 0 ):
				$status = $this->runQuery( $query );
				if ( $status == FALSE ):
					break;
				endif;
			endif;
		endforeach;

		// return
		array_pop( $this->current[ 'trace' ] );
		return $status;
	}


	/* ==================================================================================
	 * PRIVATE Function processing queries
	 *
	 * params:	$query			= sql to be executed
	 * return:	TRUE /FALSE (FALSE in case of errors )
	 * ==================================================================================
	 */
	private function runQuery( $query )
	{
		$status = TRUE;

		// update current array
		$this->current[ 'trace' ][] = __FUNCTION__;

		// placeholder handling
		$query = $this->setPlaceholders( $query );

		// run the query
		try
		{
			$db_handle  = $this->database->get_db_handle();
			$db_handle->query( $query );
			$errorInfo = $db_handle->errorInfo();

			if (( is_array( $errorInfo ) == TRUE )
			and ( isset( $errorInfo[2] ) == TRUE )):
				$error = $errorInfo[2];
				$status = FALSE;
			endif;
		}
		catch ( PDOException $e )
		{
			$error = $e->getMessage();
			$status = FALSE;
		}

		// error handling
		if ( $status == FALSE ):
			$this->writeLogDetail( array( 'log_type'	=> 'ERROR'
										, 'log_code'	=> 'SQL_ERROR'
										, 'add_info'	=> 'run query error [ ' . $error . ' ]'
										, 'content'	 	=> $query
										, 'status'		=> $status
										)
								);
		else:
			$this->writeLogDetail( array( 'log_type'	=> 'INFO'
										, 'log_code'	=> 'QUERY'
										, 'content'		=> $query
										, 'status'		=> $status
										)
								);
		endif;

		// return
		array_pop( $this->current[ 'trace' ] );
		return $status;
	}


	/* ==================================================================================
	 * PRIVATE Function placeholder Handling
	 *
	 * params:	$query
	 * return:	$query
	 * ==================================================================================
	 */
	private function setPlaceholders( $query )
	{
		// add hardcoded placeholders to those set in options
		$placeholders = $this->options[ 'PLACEHOLDERS' ];
		$placeholders[ '#NOW#' ] = '#NOW()#';
		$placeholders[ '\'#' ]   = '';
		$placeholders[ '#\'' ]   = '';

		// placeholder handling
		return str_replace( array_keys( $placeholders ), $placeholders , $query );
	}


	/* ==================================================================================
	 * PRIVATE Function to make a string out of an array
	 *
	 * params:	$where
	 * return:	$where
	 * ==================================================================================
	 */
	private function buildWhere( $where )
	{
		$return = array();
		foreach ( $where as $field => $value ):
			$item = '`' . $field . '`';
			if ( is_array( $value ) == TRUE ):
				$item .= ' IN ( '. implode( ',', $value ) . ')';
			else:
				$item .= ' = "' . $value . '"';
			endif;
			$return[] = $item;
		endforeach;

		return implode( ' AND ', $return );
	}


	/* ==================================================================================
	 * PRIVATE Function setting LOG details
	 *
	 * params:	$array	== array containg log element details
	 * return:	TRUE
	 * ==================================================================================
	 */
	private function writeLogDetail( $array )
	{
		// get current
		$trigger_code = $this->current[ 'trigger_code' ];
		$table_name = '';
		if ( array_key_exists( 'table_name', $this->current ) == TRUE ):
			$table_name = $this->current[ 'table_name' ];
		endif;

		// combine add_info data
		if ( array_key_exists( 'add_info', $array ) == TRUE ):
			$tmp = $this->current[ 'add_info' ];
			$tmp[] = $array[ 'add_info' ];
			$array[ 'add_info' ] = $tmp;
			unset( $tmp );
		endif;
		// defaults array used at least in errors list
		$fields = array( 'trigger_code' => '', 'table_name' => '', 'action' => ''
							, 'log_text' => '', 'add_info' => '', 'content' => ''
							, 'module' => $this->options[ 'MODULE' ]
							, 'start_ts' => date('c'), 'end_ts' => date('c')
						);
		// adding error text
		if (( strlen( $array[ 'log_code' ] ) > 0 )
		and ( array_key_exists( $array[ 'log_code' ], $this->options[ 'LOGTEXT' ] ) == TRUE )):
			$fields[ 'log_text' ] = $this->options[ 'LOGTEXT' ][ $array[ 'log_code' ] ];
		else:
			$fields[ 'log_text' ] = $array[ 'log_code' ];
		endif;
		// merge fields with current global data set
		// while used array_merge I got crapped data somehow.... That's why I do it like that!
		foreach ( $this->current as $field => $value ):
			if ( is_array( $value ) == TRUE ):
				$fields[ $field ] = implode( ' --> ', $value );
			else:
				$fields[ $field ] = $value;
			endif;
		endforeach;
		foreach ( $array as $field => $value ):
			if ( is_array( $value ) == TRUE ):
				$fields[ $field ] = implode( ' --> ', $value );
			else:
				$fields[ $field ] = $value;
			endif;
		endforeach;

		// prepare error list to be shown
		if ( $fields[ 'log_type' ] == 'ERROR' ):
			$this->errors[] = $fields[ 'trigger_code' ] . ' -'
							. ' ' . $fields[ 'action' ] . ' -'
							. ' ' . $fields[ 'table_name' ] . ' :'
							. ' ' . $fields[ 'log_text' ]
							. ', ' . $fields[ 'add_info' ]
							. '<br />' . $fields[ 'content' ]
							;
		endif;


		// save to logging table
		if ( $this->current[ 'logging' ] > 0 ):
			if (( strlen( trim( $fields[ 'log_type' ] ))						>  0    )
			and ( array_key_exists( $fields[ 'log_type' ], $this->log_types )	== TRUE )):
				$logId = $this->log_types[ $fields[ 'log_type' ] ];
			else:
				// not found
				$logId = self::DBS_ALL;
			endif;
			if ( $this->current[ 'logging' ] & $logId ):
				$status = $this->logger->writeLog( $fields );
				if ( $status == FALSE ):
					$errors = $this->logger->getErrors();
					foreach ( $errors as $error ):
						$this->errors[] = $error;
					endforeach;
				endif;
			endif;
		endif;

		return TRUE;
	}


	/* ==================================================================================
	 * PRIVATE Function closing the LOG
	 *
	 * return:	TRUE
	 * ==================================================================================
	 */
	private function closeLog()
	{
		// get current
		$trigger_code = $this->current[ 'trigger_code' ];

		// only when logging is ON
		if ( $this->current[ 'logging' ] > 0 ):
			// update START row with current (closing / end) timestamp
			$status = $this->logger->closeLog();
		endif;

		// wrap error list shown on trigger_code screen
		if ( count( $this->errors ) > 0 ):
			// add starting element
			array_unshift( $this->errors, '<div style="text-align: left;"><br /><br /><br /><br /><br /><br /><br />' );

			// add closing element
			$this->errors[] = '</div>';
		endif;

		return TRUE;
	}
}

?>
