<?php

/**  // € // to identify files as UTF8
 * Library providing support on creating SQLs during INSTALL, UPGRADE and UNINSTALL
 *
 * @module          lib_setuphelper
 * @author          Wolfgang Studer
 * @copyright       2016 Wolfgang Studer
 * @link            www.LEPTON-cms.org
 * @license         GNU General Public License
 * @license_terms   please see info.php of this module
 * @version         $Id: 1.04.000 | 2016-12-07
 * @lepador_link    http://www.lepton-cms.com/lepador/library/lib_setuphelper.php
 * @forum_link      http://forum.lepton-cms.org/viewtopic.php?f=66&t=3049
*/


// include class.secure.php to protect this file and the whole CMS!
	if ( defined('LEPTON_PATH') == TRUE ):
		include(LEPTON_PATH.'/framework/class.secure.php');
	else:
		$oneback = "../";
		$root = $oneback;
		$level = 1;
		while (( $level < 10 ) && ( file_exists( $root . '/framework/class.secure.php' ) == FALSE )):
			$root .= $oneback;
			$level += 1;
		endwhile;
	
		if ( file_exists($root.'/framework/class.secure.php') == TRUE ):
			include( $root . '/framework/class.secure.php' );
		else:
			trigger_error( sprintf("[ <b>%s</b> ] Can't include class.secure.php!", $_SERVER['SCRIPT_NAME']), E_USER_ERROR );
		endif;
	endif;
// end include class.secure.php


/*	**************************************
	*
	* define array with table definitions and activities to be done in
	* - install.php
	* - upgrade.php
	* - uninstall.php
	* - add.php
	* - delete.php
	*
	**************************************
*/

// init array
	$module_DB_setup = array();


/* ==================================================================================
	The array to be created contains various elements and activities for each table needed.
	Out of this array data, the activities for
	- table creation (install.php)
	- table & fields altering (upgrade.php)
	- table drop (uninstall.php)
	- adding rows (install.php, add.php)
	- deleting rows (delete.php)
	are taken and executed.

	The array contains the following elements:
	- Level 1:
		* trigger_codes per calling module
			+ INSTALL
			+ UPGRADE
			+ UNINSTALL
			+ ADD
			+ DELETE
		* tables for option settings
	- Level 2:
		* for trigger_codes: $table is the DB table you want to use.
		* for tables the option settings
			+ index
			+ options
			+ fields
	- Level 3:
		* actions (in uppercase)
		* flags (in lowercase)

	INSTALL & UPGRADE are sharing the same handling.
	- When table not exists, the table is beeing created.
	- When table exists, it is checked if tabled & fields needs to be altered.

	Sometimes you need to refer to values like page_id or section_id during execution.
	Due to, placeholders can be used instead of real values in this definition below,
	which are then set before calling the DB_setup class.
	Examples of placeholders are:
	- #PAGE_ID#
	- #SECTION_ID#
	- #USER_ID#
	Which placeholders you use and how you name it is up to you.
	An example call for building the placeholder array is like:
		// define placeholders
		$placeholders = array(    '#PAGE_ID#'		=> ( isset( $page_id ) == TRUE ? $page_id : 0 )
								, '#SECTION_ID#'	=> ( isset( $section_id ) == TRUE ? $section_id : 0 )
								, '#USER_ID#'		=> ( isset( $admin ) == TRUE ? $admin->get_user_id() : 0 )
								);

	A specific hardcoded placeholder is #NULL#, which is replaced by NULL.
	Reason of this placeholder is that you can not assign NULL value into an array and use it later accordingly.


	Note: If a trigger_code is NOT DEFINED in the setup array at all, then nothing is executed for this trigger_code. No default actions!!!
		Also if trigger_code is defined with an empty table level array. Nothing is done.


	In detail, the following actions & flags are possible per trigger_code & table:
	- INSTALL, UPGRADE, UNINSTALL, ADD, DELETE
		* logging	= omitted ( = default logging)
				This flag allows overwriting for curent table of default logging.
				But ... When default logging is switched off,this flag is ignored.
				Only when default logging is ON, you can switch it OFF with this setting.
				Logging can be set on various levels, on any combination or for all.
				0 = no logging
				1 = errors
				4 = infos
				5 = errors & infos
				8 = notice
				9 = errors & notice
				12 = infos & notice
				13 = all

	- ADD
		* DO	= Same logic & rules as for DELETE - DO, see below.
				Only difference is the default "insert" for REQUEST.

	- DELETE
		* DO	= array( array( request = array(..), ... ), array( request = array(..), ... ) , .. );
			==>	contain 1 or more conditions array to be used to delete rows.
				A condition contains 1 to 3 request elements and the details per request.
				For REQUEST, FIELDS and WHERE see also:
				https://doc.lepton-cms.org/docu/english/home/core/new-methods/build_mysql_query.php
				Following is possible
				+ REQUEST	= one of "insert", "update", "delete" (default)
					==>		Example: 'REQUEST' => 'delete'
				+ TABLE		= name of table to be used
					==>		Example: 'my_table_name'
							Default is current table ($table)
							Not needed when action is to be done on current table.
							Normally it is not needed in ADD/DELETE DO actions but might be needed in PRE & POST actions.
				+ FIELDS	= list of fields in an array
					==>		Example: 'FIELDS' => array( 'fld1', 'fld2')
							Not needed with REQUEST = delete.
							Mandatory with REQUEST = "insert", "update".
				+ WHERE		= associative array of fields = value pairs or a string containg WHERE condition
					==>		Example: 'WHERE' => array( 'fld1' => 'val1', 'fld2' => 'val2')
					==>		Example: 'WHERE' =>'`fld1` = "val1" OR `fld2` = "val2"'
							Mandatory with REQUEST = delete.
							Array pairs are combined with AND.
							When element field->value is an array, value items are combined with "IN( ..)",
							otherwise with " = ".
					==>		Example: 'WHERE' => array( 'fld1' => 'val1', 'fld2' => array( 'val2', 'val3' ))
					==>		Generate: WHERE `fld1` = "val1" AND `fld2` IN ( "val2","val3" )
				+ QUERY		= a string containg the full &valid sql to be executed.
					==>		Example: 'QUERY' => 'DELETE FROM `mytable` WHERE `myfield`< 1;'
							Note that this item is used only if REQUEST, FIELDS and WHERE have not overwritten it by a build query!
			==>	Full example: delete table rows belonging to current section
				$module_DB_setup[ 'DELETE' ][ $table ][ 'DO' ][]	= array( 'WHERE'	=> array(
												  'section_id' => '#SECTION_ID#'
												)
											);

	- INSTALL, UPGRADE, UNINSTALL
		* DO	= action: TRUE or FALSE or omitted ( = TRUE)
			==>	defines if this trigger_code is to be processed.
			==>	Full example: Do not process UPGRADE for current table
				$module_DB_setup[ 'UPGRADE' ][ $table ][ 'DO' ]	= FALSE;

	- INSTALL, UNINSTALL
		* drop	= flag: TRUE or FALSE or omitted (= TRUE)
			==>	defines if table is to be dropped when current trigger_code is processed.
				Per default DROP will be FALSE during UPGRADE and TRUE on INSTALL & UNINSTALL.
				Note that DROP action is not executed if DO action is false.
			==>	Full example: Do not drop table during uninstall
				$module_DB_setup[ 'UNINSTALL' ][ $table ][ 'drop' ]	= FALSE;

	- UPGRADE
		* drop	= flag: TRUE or FALSE or omitted (= FALSE)
			==>	defines if table is to be dropped when UPGRADE trigger_code is processed.
				Per default DROP will be false during UPGRADE and TRUE on INSTALL & UNINSTALL.
				Note that DROP action is not executed if DO action is false.
			==>	Full example: Drop table during upgrade
				$module_DB_setup[ 'UPGRADE' ][ $table ][ 'drop' ]	= TRUE;

	- INSTALL, UPGRADE
		* alter	= flag: TRUE or FALSE or omitted (= TRUE)

		Note:	When a table not exists, the table will be created.
				But if the table already exists, regardless whether in INSTALL or UPGRADE trigger_code,
				the system tries to alter the table if needed.
				The implementation of altering tables is simple and straight forward ....
				A temp table with defined setup (fields, types, index, options, ...) is created
				and compared with the result of "SHOW CREATE TABLE" SQL function against both old & new table.
				When there are no differences, the temp table is deleted.
				When there are differences (e.g. even in engine or charset or ... ), then ..
				- the data from existing table are taken over into new temp table
				- the existing table is renamed (table name _BAK_timestamp) or deleted (see bak_copy below)
				- the temp table is renamed to default table name
				With alter = FALSE, this functionality can be switched OFF, however then there is then no alter table at all!
			==>	Full example: Do not alter the table
				$module_DB_setup[ 'UPGRADE' ][ $table ][ 'alter' ]	= FALSE;

		* bak_copy	= flag: TRUE or FALSE or omitted (= TRUE)
			==>	defines if a backup copy of a table should be done within alter table process.
				This backup copy flag "bak_copy" is related to the alter flag.
				Only when alter flag is TRUE (or omitted) and a change has been identified
				which results into a alter table statement, then the bak_copy flag is checked.
				When FALSE, the existing table is dropped after data copy.
				When TRUE (or omitted), the existing table is renamed (table name _BAK_timestamp).
			==>	Full example: Do not create backup copy
				$module_DB_setup[ 'UPGRADE' ][ $table ][ 'bak_copy' ]	= FALSE;

	- INSTALL, UPGRADE, UNINSTALL, ADD, DELETE
		* PRE	= array of sql to be executed BEFORE the standard action of the trigger_code.
					Same logic & rules as for DELETE - DO, see above.
					Only difference is that there is no default for REQUEST.
					Note that PRE action is processed only if DO action is TRUE or set.
		* POST	= array of sql to be executed AFTER the standard action of the trigger_code
					Same logic & rules as for DELETE - DO, see above.
					Only difference is that there is no default for REQUEST.
					Note that POST action is processed only if DO action is TRUE or set.

	- INSTALL, UPGRADE (used for this trigger_codes only but defined on $table of array root !!)
		* index		= array of table index
			==>		Full Example: addingle single index
					$module_DB_setup[ $table ][ 'index' ][]		= 'PRIMARY KEY (`text_id`)';
			==>		Full Example: addingle multiple index
					$module_DB_setup[ $table ][ 'index' ][]		= 'PRIMARY KEY (`text_id`)';
					$module_DB_setup[ $table ][ 'index' ][]		= 'UNIQUE KEY `languagepath` (`language`,`path`)';
		* options	= array of table options.
						See 'table_option:' in: http://dev.mysql.com/doc/refman/5.7/en/create-table.html
			==>		Example: adding a table comment
					$module_DB_setup[ $table ][ 'options' ][]	= 'COMMENT "my table comment"';
		* fields	= array of field elements
					syntax: 'fields' = array( array( ... ), array( ... ), ... )
					The number of inner arrays depends on the number of fields for current table.
					The items inside the inner array are:
					+ 'Field'	=> 'field name' (*)
					+ 'Type'	=> 'type and length' (*)		// INT(11), TINYINT, CHAR(3), ..
					+ 'Null'	=> 'NO'							// NO = NOT NULL, YES = NULL (default)
					+ 'Default'	=> '"A"'						// double quote strings!
					+ 'Key'		=> ''							// 'PRI' on primary key fields, otherwise empty
					+ 'Options'	=> 'COMMENT "field comment"'	// a string of field options
					+ 'Convert'	=> array( 'Source' => 'source field', 'Rule' => 'the convertion rule', 'onType' => 'condition with old type' )
					(*) The 2 items 'Field' and 'Type' are mandatory.
					The array key names are same as returned from describe_table method.
					See also: https://doc.lepton-cms.org/docu/english/home/core/new-methods/describe_table.php
					Note that the sequence of the items inside the array does not matter.
					But the sequence of the arrays inside the 'fields' array matters and defines the sequence of the fields in the table !

					The 'Convert' is used only when data needs to be copied from old table to new table.
					When 'Convert' is not available, the copy uses the 'Field' value for source and target field.
					When 'Convert' is available, the value of 'Content->Source' is used as source, the 'Field' value as target.
					Due to, the 'Convert->Source' value can be used for field renaming.
					With the additional fields "Rule" and "onType" you can define data convertion logic.
			==>		Example: Convert old INT(11) based value to new Timestamp
						'Field' => 'modified_when'
						, ...
						, 'Convert' => array( 'Source' => 'modified_when', 'Rule' => 'FROM_UNIXTIME(`modified_when`)' )
			==>		Example: Convert old INT(11) based value to new Timestamp but only if old type is 'INT(11)' including renaming
						'Field' => 'modified_when'
						, ...
						, 'Convert' => array( 'Source' => 'modified_at', 'onType' => 'int(11)', 'Rule' => 'FROM_UNIXTIME(`modified_when`)' )
			==>		Example: Rename old field 'modified_at' to 'modified_when' without data conversion
						'Field' => 'modified_when'
						, ...
						, 'Convert' => array( 'Source' => 'modified_at' )

 * ==================================================================================
 */

/*	**************************************
	*
	* module table: foldergallery2_jq_settings
	*
	**************************************
*/
	$table = TABLE_PREFIX . 'mod_foldergallery2_jq_settings';
	$module_DB_setup[ $table ][ 'index' ][]		= 'PRIMARY KEY (`page_id`,`section_id`)';
	$module_DB_setup[ $table ][ 'options' ][]	= 'COMMENT "Table for foldergallery settings"';
	$module_DB_setup[ $table ][ 'fields' ]		= array(
			  array(
				  'Field'	=> 'page_id'
				, 'Type'	=> 'INT(9)'
				, 'Null'	=> 'NO'
				, 'Key'		=> 'PRI'
				, 'Options'	=> 'COMMENT "Primary Key"'
				)
			, array(
				  'Field'	=> 'section_id'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Key'		=> 'PRI'
				, 'Options'	=> 'COMMENT "Primary Key"'
				)
			, array(
				  'Field'	=> 'created_when'
				, 'Type'	=> 'TIMESTAMP'
				, 'Null'	=> 'NO'
				, 'Default'	=> '"0000-00-00 00:00:00"'
				, 'Convert' => array( 'Source' => 'created_when', 'onType' => 'int(11)', 'Rule' => 'FROM_UNIXTIME(`created_when`)' )
				)
			, array(
				  'Field'	=> 'modified_when'
				, 'Type'	=> 'TIMESTAMP'
				, 'Null'	=> 'NO'
				, 'Default'	=> 'CURRENT_TIMESTAMP'
				, 'Extra'	=> 'ON UPDATE CURRENT_TIMESTAMP'
				, 'Convert' => array( 'Source' => 'modified_when', 'onType' => 'int(11)', 'Rule' => 'FROM_UNIXTIME(`modified_when`)' )
				)
			, array(
				  'Field'	=> 'modified_by'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'version_number'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'sync_status'
				, 'Type'	=> 'VARCHAR(20)'
				)
			, array(
				  'Field'	=> 'last_sync'
				, 'Type'	=> 'INT(11)'
				)
			, array(
				  'Field'	=> 'root_dir'
				, 'Type'	=> 'VARCHAR(255)'
				, 'Null'	=> 'NO'
				)
			, array(
				  'Field'	=> 'root_dir_old'
				, 'Type'	=> 'VARCHAR(255)'
				)
			, array(
				  'Field'	=> 'extensions'
				, 'Type'	=> 'VARCHAR(255)'
				, 'Null'	=> 'NO'
				)
			, array(
				  'Field'	=> 'exclude_dirs'
				, 'Type'	=> 'VARCHAR(255)'
				)
			, array(
				  'Field'	=> 'thumbnail_fixdir'
				, 'Type'	=> 'VARCHAR(50)'
				)
			, array(
				  'Field'	=> 'thumbnail_dir'
				, 'Type'	=> 'VARCHAR(50)'
				, 'Null'	=> 'NO'
				)
			, array(
				  'Field'	=> 'thumbnail_dir_old'
				, 'Type'	=> 'VARCHAR(50)'
				)
			, array(
				  'Field'	=> 'thumb_size'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				)
			, array(
				  'Field'	=> 'thumb_position'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				)
			, array(
				  'Field'	=> 'file_size_limit'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '10'
				)
			, array(
				  'Field'	=> 'ratio'
				, 'Type'	=> 'VARCHAR(10)'
				, 'Null'	=> 'NO'
				)
			, array(
				  'Field'	=> 'ratio_x'
				, 'Type'	=> 'INT(11)'
				)
			, array(
				  'Field'	=> 'ratio_y'
				, 'Type'	=> 'INT(11)'
				)
			, array(
				  'Field'	=> 'preset_name'
				, 'Type'	=> 'VARCHAR(100)'
				)
			, array(
				  'Field'	=> 'preset_path'
				, 'Type'	=> 'VARCHAR(255)'
				)
			, array(
				  'Field'	=> 'pics_pp'
				, 'Type'	=> 'SMALLINT'
				)
			, array(
				  'Field'	=> 'catpic'
				, 'Type'	=> 'TINYINT(4)'
				)
			, array(
				  'Field'	=> 'sort_frontend'
				, 'Type'	=> 'TINYINT(4)'
				)
			, array(
				  'Field'	=> 'options'
				, 'Type'	=> 'TEXT'
				)
		);

	$module_DB_setup[ 'INSTALL' ][ $table ][ 'DO' ]		= TRUE;
	$module_DB_setup[ 'INSTALL' ][ $table ][ 'POST' ][]	= array(
											  'REQUEST'	=> 'INSERT'
											, 'FIELDS'	=> array(
												  'page_id'			=> 0
												, 'section_id'		=> 0
												, 'created_when'	=> '#NOW#'
												, 'modified_by'		=> '#USER_ID#'
												, 'root_dir'		=> '"0"'
												, 'extensions'		=> '"0"'
												, 'thumbnail_dir'	=> '"0"'
												, 'thumb_size'		=> 0
												, 'thumb_position'	=> 0
												, 'file_size_limit'	=> 0
												, 'ratio'			=> '"0"'
												)
											);

	$module_DB_setup[ 'UPGRADE' ][ $table ][ 'DO' ]		= TRUE;

	$module_DB_setup[ 'UNINSTALL' ][ $table ][ 'DO' ]	= TRUE;

	$module_DB_setup[ 'DELETE' ][ $table ][ 'DO' ][]	= array( 'WHERE'	=> array(
												  'section_id' => '#SECTION_ID#'
												)
											);


/*	**************************************
	*
	* module table: foldergallery2_jq_files
	*
	**************************************
*/
	$table = TABLE_PREFIX . 'mod_foldergallery2_jq_files';
	$module_DB_setup[ $table ][ 'index' ][]		= 'PRIMARY KEY (`file_id`)';
	$module_DB_setup[ $table ][ 'options' ][]	= 'COMMENT "Table for foldergallery images"';
	$module_DB_setup[ $table ][ 'fields' ]		= array(
			  array(
				  'Field'	=> 'file_id'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Extra'	=> 'AUTO_INCREMENT'
				, 'Key'		=> 'PRI'
				)
			, array(
				  'Field'	=> 'created_when'
				, 'Type'	=> 'TIMESTAMP'
				, 'Null'	=> 'NO'
				, 'Default'	=> '"0000-00-00 00:00:00"'
				, 'Convert' => array( 'Source' => 'created_when', 'onType' => 'int(11)', 'Rule' => 'FROM_UNIXTIME(`created_when`)' )
				)
			, array(
				  'Field'	=> 'modified_when'
				, 'Type'	=> 'TIMESTAMP'
				, 'Null'	=> 'NO'
				, 'Default'	=> 'CURRENT_TIMESTAMP'
				, 'Extra'	=> 'ON UPDATE CURRENT_TIMESTAMP'
				, 'Convert' => array( 'Source' => 'modified_when', 'onType' => 'int(11)', 'Rule' => 'FROM_UNIXTIME(`modified_when`)' )
				)
			, array(
				  'Field'	=> 'modified_by'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'version_number'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'page_id'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'section_id'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'parent_id'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'kind'
				, 'Type'	=> 'TINYINT(4)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'active'
				, 'Type'	=> 'TINYINT(4)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '1'
				)
			, array(
				  'Field'	=> 'position'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'catpic_mark'
				, 'Type'	=> 'TINYINT(4)'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'file_name'
				, 'Type'	=> 'VARCHAR(255)'
				)
			, array(
				  'Field'	=> 'path'
				, 'Type'	=> 'VARCHAR(255)'
				)
			, array(
				  'Field'	=> 'md5_checksum'
				, 'Type'	=> 'VARCHAR(50)'
				)
			, array(
				  'Field'	=> 'process_status'
				, 'Type'	=> 'TINYINT(4)'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'attributes'
				, 'Type'	=> 'TEXT'
				)
		);

	$module_DB_setup[ 'INSTALL' ][ $table ][ 'DO' ]		= TRUE;

	$module_DB_setup[ 'UPGRADE' ][ $table ][ 'DO' ]		= TRUE;

	$module_DB_setup[ 'UNINSTALL' ][ $table ][ 'DO' ]	= TRUE;

	$module_DB_setup[ 'DELETE' ][ $table ][ 'DO' ][]	= array( 'WHERE'	=> array(
												  'section_id' => '#SECTION_ID#'
												)
											);


/*	**************************************
	*
	* module table: foldergallery2_jq_categories
	*
	**************************************
*/
	$table = TABLE_PREFIX . 'mod_foldergallery2_jq_categories';
	$module_DB_setup[ $table ][ 'index' ][]		= 'PRIMARY KEY (`category_id`)';
	$module_DB_setup[ $table ][ 'options' ][]	= 'COMMENT "Table for foldergallery categories aka folders"';
	$module_DB_setup[ $table ][ 'fields' ]		= array(
			  array(
				  'Field'	=> 'category_id'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Extra'	=> 'AUTO_INCREMENT'
				, 'Key'		=> 'PRI'
				)
			, array(
				  'Field'	=> 'created_when'
				, 'Type'	=> 'TIMESTAMP'
				, 'Null'	=> 'NO'
				, 'Default'	=> '"0000-00-00 00:00:00"'
				, 'Convert' => array( 'Source' => 'created_when', 'onType' => 'int(11)', 'Rule' => 'FROM_UNIXTIME(`created_when`)' )
				)
			, array(
				  'Field'	=> 'modified_when'
				, 'Type'	=> 'TIMESTAMP'
				, 'Null'	=> 'NO'
				, 'Default'	=> 'CURRENT_TIMESTAMP'
				, 'Extra'	=> 'ON UPDATE CURRENT_TIMESTAMP'
				, 'Convert' => array( 'Source' => 'modified_when', 'onType' => 'int(11)', 'Rule' => 'FROM_UNIXTIME(`modified_when`)' )
				)
			, array(
				  'Field'	=> 'modified_by'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'version_number'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'page_id'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'section_id'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'parent_id'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'category'
				, 'Type'	=> 'VARCHAR(255)'
				)
			, array(
				  'Field'	=> 'parent'
				, 'Type'	=> 'VARCHAR(255)'
				)
			, array(
				  'Field'	=> 'active'
				, 'Type'	=> 'TINYINT(4)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '1'
				)
			, array(
				  'Field'	=> 'is_empty'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '1'
				)
			, array(
				  'Field'	=> 'position'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'niveau'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '1'
				)
			, array(
				  'Field'	=> 'has_child'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'childs'
				, 'Type'	=> 'VARCHAR(255)'
				)
			, array(
				  'Field'	=> 'catpic'
				, 'Type'	=> 'TINYINT(4)'
				)
			, array(
				  'Field'	=> 'sort_frontend'
				, 'Type'	=> 'TINYINT(4)'
				)
			, array(
				  'Field'	=> 'path'
				, 'Type'	=> 'VARCHAR(255)'
				)
			, array(
				  'Field'	=> 'process_status'
				, 'Type'	=> 'TINYINT(4)'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'attributes'
				, 'Type'	=> 'TEXT'
				)
		);

	$module_DB_setup[ 'INSTALL' ][ $table ][ 'DO' ]		= TRUE;

	$module_DB_setup[ 'UPGRADE' ][ $table ][ 'DO' ]		= TRUE;

	$module_DB_setup[ 'UNINSTALL' ][ $table ][ 'DO' ]	= TRUE;

	$module_DB_setup[ 'DELETE' ][ $table ][ 'DO' ][]	= array( 'WHERE'	=> array(
												  'section_id' => '#SECTION_ID#'
												)
											);


/*	**************************************
	*
	* module table: foldergallery2_jq_text
	*
	**************************************
*/
	$table = TABLE_PREFIX . 'mod_foldergallery2_jq_text';
	$module_DB_setup[ $table ][ 'index' ][]		= 'PRIMARY KEY (`text_id`)';
	$module_DB_setup[ $table ][ 'index' ][]		= 'UNIQUE KEY `languagepath` (`language`,`path`)';
	$module_DB_setup[ $table ][ 'options' ][]	= 'COMMENT "Table for foldergallery text for images and categories"';
	$module_DB_setup[ $table ][ 'fields' ]		= array(
			array(
				  'Field'	=> 'text_id'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Extra'	=> 'AUTO_INCREMENT'
				, 'Key'		=> 'PRI'
				)
			, array(
				  'Field'	=> 'created_when'
				, 'Type'	=> 'TIMESTAMP'
				, 'Null'	=> 'NO'
				, 'Default'	=> '"0000-00-00 00:00:00"'
				, 'Convert' => array( 'Source' => 'created_when', 'onType' => 'int(11)', 'Rule' => 'FROM_UNIXTIME(`created_when`)' )
				)
			, array(
				  'Field'	=> 'modified_when'
				, 'Type'	=> 'TIMESTAMP'
				, 'Null'	=> 'NO'
				, 'Default'	=> 'CURRENT_TIMESTAMP'
				, 'Extra'	=> 'ON UPDATE CURRENT_TIMESTAMP'
				, 'Convert' => array( 'Source' => 'modified_when', 'onType' => 'int(11)', 'Rule' => 'FROM_UNIXTIME(`modified_when`)' )
				)
			, array(
				  'Field'	=> 'modified_by'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'version_number'
				, 'Type'	=> 'INT(11)'
				, 'Null'	=> 'NO'
				, 'Default'	=> '0'
				)
			, array(
				  'Field'	=> 'language'
				, 'Type'	=> 'CHAR(3)'
				, 'Null'	=> 'NO'
				, 'Key'		=> ''
				)
			, array(
				  'Field'	=> 'path'
				, 'Type'	=> 'VARCHAR(255)'
				, 'Null'	=> 'NO'
				, 'Key'		=> ''
				)
			, array(
				  'Field'	=> 'caption'
				, 'Type'	=> 'TEXT'
				)
			, array(
				  'Field'	=> 'keyword_1'
				, 'Type'	=> 'VARCHAR(255)'
				)
			, array(
				  'Field'	=> 'keyword_2'
				, 'Type'	=> 'VARCHAR(255)'
				)
			, array(
				  'Field'	=> 'keyword_3'
				, 'Type'	=> 'VARCHAR(255)'
				)
			, array(
				  'Field'	=> 'description'
				, 'Type'	=> 'TEXT'
				)
			, array(
				  'Field'	=> 'process_status'
				, 'Type'	=> 'TINYINT(4)'
				, 'Default'	=> '0'
				)
		);

	$module_DB_setup[ 'INSTALL' ][ $table ][ 'DO' ]		= TRUE;

	$module_DB_setup[ 'UPGRADE' ][ $table ][ 'DO' ]		= TRUE;

	$module_DB_setup[ 'UNINSTALL' ][ $table ][ 'DO' ]	= TRUE;

/*	// no deletion of this table during DELETE !
	$module_DB_setup[ 'DELETE' ][ $table ][ 'DO' ][]	= array( 'WHERE'	=> array(
												  'section_id' => '#SECTION_ID#'
												)
											);
*/


?>
